# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: constraints.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import messages_pb2 as messages__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x63onstraints.proto\x1a\x0emessages.proto\"\xbf\x01\n\x14SummaryConstraintMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x66irst_field\x18\x02 \x01(\t\x12\x16\n\x0csecond_field\x18\x03 \x01(\tH\x00\x12\x0f\n\x05value\x18\x04 \x01(\x01H\x00\x12/\n\x07\x62\x65tween\x18\x05 \x01(\x0b\x32\x1c.SummaryBetweenConstraintMsgH\x00\x12\x0f\n\x02op\x18\x06 \x01(\x0e\x32\x03.Op\x12\x0f\n\x07verbose\x18\x07 \x01(\x08\x42\x08\n\x06second\"\x8c\x01\n\x1bSummaryBetweenConstraintMsg\x12\x16\n\x0csecond_field\x18\x01 \x01(\tH\x00\x12\x15\n\x0blower_value\x18\x02 \x01(\x01H\x00\x12\x15\n\x0bthird_field\x18\x03 \x01(\tH\x01\x12\x15\n\x0bupper_value\x18\x04 \x01(\x01H\x01\x42\x07\n\x05lowerB\x07\n\x05upper\"~\n\x12ValueConstraintMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x05value\x18\x02 \x01(\x01H\x00\x12\x17\n\rregex_pattern\x18\x05 \x01(\tH\x00\x12\x0f\n\x02op\x18\x03 \x01(\x0e\x32\x03.Op\x12\x0f\n\x07verbose\x18\x04 \x01(\x08\x42\x0e\n\x0csecond_field\"?\n\x13ValueConstraintMsgs\x12(\n\x0b\x63onstraints\x18\x01 \x03(\x0b\x32\x13.ValueConstraintMsg\"C\n\x15SummaryConstraintMsgs\x12*\n\x0b\x63onstraints\x18\x01 \x03(\x0b\x32\x15.SummaryConstraintMsg\"\xf4\x02\n\x14\x44\x61tasetConstraintMsg\x12&\n\nproperties\x18\x01 \x01(\x0b\x32\x12.DatasetProperties\x12\x46\n\x11value_constraints\x18\x02 \x03(\x0b\x32+.DatasetConstraintMsg.ValueConstraintsEntry\x12J\n\x13summary_constraints\x18\x03 \x03(\x0b\x32-.DatasetConstraintMsg.SummaryConstraintsEntry\x1aM\n\x15ValueConstraintsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.ValueConstraintMsgs:\x02\x38\x01\x1aQ\n\x17SummaryConstraintsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12%\n\x05value\x18\x02 \x01(\x0b\x32\x16.SummaryConstraintMsgs:\x02\x38\x01*b\n\x02Op\x12\n\n\x06unused\x10\x00\x12\x06\n\x02LT\x10\x01\x12\x06\n\x02LE\x10\x02\x12\x06\n\x02\x45Q\x10\x03\x12\x06\n\x02NE\x10\x04\x12\x06\n\x02GE\x10\x05\x12\x06\n\x02GT\x10\x06\x12\t\n\x05MATCH\x10\x07\x12\x0b\n\x07NOMATCH\x10\x08\x12\x08\n\x04\x42TWN\x10\tB,\n\x1b\x63om.whylogs.core.constraintB\x0b\x43onstraintsP\x01\x62\x06proto3')

_OP = DESCRIPTOR.enum_types_by_name['Op']
Op = enum_type_wrapper.EnumTypeWrapper(_OP)
unused = 0
LT = 1
LE = 2
EQ = 3
NE = 4
GE = 5
GT = 6
MATCH = 7
NOMATCH = 8
BTWN = 9


_SUMMARYCONSTRAINTMSG = DESCRIPTOR.message_types_by_name['SummaryConstraintMsg']
_SUMMARYBETWEENCONSTRAINTMSG = DESCRIPTOR.message_types_by_name['SummaryBetweenConstraintMsg']
_VALUECONSTRAINTMSG = DESCRIPTOR.message_types_by_name['ValueConstraintMsg']
_VALUECONSTRAINTMSGS = DESCRIPTOR.message_types_by_name['ValueConstraintMsgs']
_SUMMARYCONSTRAINTMSGS = DESCRIPTOR.message_types_by_name['SummaryConstraintMsgs']
_DATASETCONSTRAINTMSG = DESCRIPTOR.message_types_by_name['DatasetConstraintMsg']
_DATASETCONSTRAINTMSG_VALUECONSTRAINTSENTRY = _DATASETCONSTRAINTMSG.nested_types_by_name['ValueConstraintsEntry']
_DATASETCONSTRAINTMSG_SUMMARYCONSTRAINTSENTRY = _DATASETCONSTRAINTMSG.nested_types_by_name['SummaryConstraintsEntry']
SummaryConstraintMsg = _reflection.GeneratedProtocolMessageType('SummaryConstraintMsg', (_message.Message,), {
  'DESCRIPTOR' : _SUMMARYCONSTRAINTMSG,
  '__module__' : 'constraints_pb2'
  # @@protoc_insertion_point(class_scope:SummaryConstraintMsg)
  })
_sym_db.RegisterMessage(SummaryConstraintMsg)

SummaryBetweenConstraintMsg = _reflection.GeneratedProtocolMessageType('SummaryBetweenConstraintMsg', (_message.Message,), {
  'DESCRIPTOR' : _SUMMARYBETWEENCONSTRAINTMSG,
  '__module__' : 'constraints_pb2'
  # @@protoc_insertion_point(class_scope:SummaryBetweenConstraintMsg)
  })
_sym_db.RegisterMessage(SummaryBetweenConstraintMsg)

ValueConstraintMsg = _reflection.GeneratedProtocolMessageType('ValueConstraintMsg', (_message.Message,), {
  'DESCRIPTOR' : _VALUECONSTRAINTMSG,
  '__module__' : 'constraints_pb2'
  # @@protoc_insertion_point(class_scope:ValueConstraintMsg)
  })
_sym_db.RegisterMessage(ValueConstraintMsg)

ValueConstraintMsgs = _reflection.GeneratedProtocolMessageType('ValueConstraintMsgs', (_message.Message,), {
  'DESCRIPTOR' : _VALUECONSTRAINTMSGS,
  '__module__' : 'constraints_pb2'
  # @@protoc_insertion_point(class_scope:ValueConstraintMsgs)
  })
_sym_db.RegisterMessage(ValueConstraintMsgs)

SummaryConstraintMsgs = _reflection.GeneratedProtocolMessageType('SummaryConstraintMsgs', (_message.Message,), {
  'DESCRIPTOR' : _SUMMARYCONSTRAINTMSGS,
  '__module__' : 'constraints_pb2'
  # @@protoc_insertion_point(class_scope:SummaryConstraintMsgs)
  })
_sym_db.RegisterMessage(SummaryConstraintMsgs)

DatasetConstraintMsg = _reflection.GeneratedProtocolMessageType('DatasetConstraintMsg', (_message.Message,), {

  'ValueConstraintsEntry' : _reflection.GeneratedProtocolMessageType('ValueConstraintsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETCONSTRAINTMSG_VALUECONSTRAINTSENTRY,
    '__module__' : 'constraints_pb2'
    # @@protoc_insertion_point(class_scope:DatasetConstraintMsg.ValueConstraintsEntry)
    })
  ,

  'SummaryConstraintsEntry' : _reflection.GeneratedProtocolMessageType('SummaryConstraintsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETCONSTRAINTMSG_SUMMARYCONSTRAINTSENTRY,
    '__module__' : 'constraints_pb2'
    # @@protoc_insertion_point(class_scope:DatasetConstraintMsg.SummaryConstraintsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETCONSTRAINTMSG,
  '__module__' : 'constraints_pb2'
  # @@protoc_insertion_point(class_scope:DatasetConstraintMsg)
  })
_sym_db.RegisterMessage(DatasetConstraintMsg)
_sym_db.RegisterMessage(DatasetConstraintMsg.ValueConstraintsEntry)
_sym_db.RegisterMessage(DatasetConstraintMsg.SummaryConstraintsEntry)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.whylogs.core.constraintB\013ConstraintsP\001'
  _DATASETCONSTRAINTMSG_VALUECONSTRAINTSENTRY._options = None
  _DATASETCONSTRAINTMSG_VALUECONSTRAINTSENTRY._serialized_options = b'8\001'
  _DATASETCONSTRAINTMSG_SUMMARYCONSTRAINTSENTRY._options = None
  _DATASETCONSTRAINTMSG_SUMMARYCONSTRAINTSENTRY._serialized_options = b'8\001'
  _OP._serialized_start=1011
  _OP._serialized_end=1109
  _SUMMARYCONSTRAINTMSG._serialized_start=38
  _SUMMARYCONSTRAINTMSG._serialized_end=229
  _SUMMARYBETWEENCONSTRAINTMSG._serialized_start=232
  _SUMMARYBETWEENCONSTRAINTMSG._serialized_end=372
  _VALUECONSTRAINTMSG._serialized_start=374
  _VALUECONSTRAINTMSG._serialized_end=500
  _VALUECONSTRAINTMSGS._serialized_start=502
  _VALUECONSTRAINTMSGS._serialized_end=565
  _SUMMARYCONSTRAINTMSGS._serialized_start=567
  _SUMMARYCONSTRAINTMSGS._serialized_end=634
  _DATASETCONSTRAINTMSG._serialized_start=637
  _DATASETCONSTRAINTMSG._serialized_end=1009
  _DATASETCONSTRAINTMSG_VALUECONSTRAINTSENTRY._serialized_start=849
  _DATASETCONSTRAINTMSG_VALUECONSTRAINTSENTRY._serialized_end=926
  _DATASETCONSTRAINTMSG_SUMMARYCONSTRAINTSENTRY._serialized_start=928
  _DATASETCONSTRAINTMSG_SUMMARYCONSTRAINTSENTRY._serialized_end=1009
# @@protoc_insertion_point(module_scope)



import logging
import re
from typing import List, Mapping, Optional

from google.protobuf.json_format import Parse

from whylogs.proto import (
    DatasetConstraintMsg,
    DatasetProperties,
    NumberSummary,
    Op,
    SummaryBetweenConstraintMsg,
    SummaryConstraintMsg,
    SummaryConstraintMsgs,
    ValueConstraintMsg,
    ValueConstraintMsgs,
)
from whylogs.util.protobuf import message_to_json

logger = logging.getLogger(__name__)

"""
Dict indexed by constraint operator.

These help translate from constraint schema to language-specific functions that are faster to evaluate.
This is just a form of currying, and I chose to bind the boolean comparison operator first.
"""
_value_funcs = {
    # functions that compare an incoming feature value to a literal value.
    Op.LT: lambda x: lambda v: v < x,  # assert incoming value 'v' is less than some fixed value 'x'
    Op.LE: lambda x: lambda v: v <= x,
    Op.EQ: lambda x: lambda v: v == x,
    Op.NE: lambda x: lambda v: v != x,
    Op.GE: lambda x: lambda v: v >= x,
    Op.GT: lambda x: lambda v: v > x,  # assert incoming value 'v' is greater than some fixed value 'x'
    Op.MATCH: lambda x: lambda v: x.match(v) is not None,
    Op.NOMATCH: lambda x: lambda v: x.match(v) is None,
}

_summary_funcs1 = {
    # functions that compare a summary field to a literal value.
    Op.LT: lambda f, v: lambda s: getattr(s, f) < v,
    Op.LE: lambda f, v: lambda s: getattr(s, f) <= v,
    Op.EQ: lambda f, v: lambda s: getattr(s, f) == v,
    Op.NE: lambda f, v: lambda s: getattr(s, f) != v,
    Op.GE: lambda f, v: lambda s: getattr(s, f) >= v,
    Op.GT: lambda f, v: lambda s: getattr(s, f) > v,
    Op.BTWN: lambda f, v1, v2: lambda s: v1 <= getattr(s, f) <= v2,
}

_summary_funcs2 = {
    # functions that compare two summary fields.
    Op.LT: lambda f, f2: lambda s: getattr(s, f) < getattr(s, f2),
    Op.LE: lambda f, f2: lambda s: getattr(s, f) <= getattr(s, f2),
    Op.EQ: lambda f, f2: lambda s: getattr(s, f) == getattr(s, f2),
    Op.NE: lambda f, f2: lambda s: getattr(s, f) != getattr(s, f2),
    Op.GE: lambda f, f2: lambda s: getattr(s, f) >= getattr(s, f2),
    Op.GT: lambda f, f2: lambda s: getattr(s, f) > getattr(s, f2),
    Op.BTWN: lambda f, f2, f3: lambda s: getattr(s, f2) <= getattr(s, f) <= getattr(s, f3),
}


class ValueConstraint:
    """
    ValueConstraints express a binary boolean relationship between an implied numeric value and a literal.
    When associated with a ColumnProfile, the relation is evaluated for every incoming value that is processed by whylogs.

    Parameters
    ----------
    op : whylogs.proto.Op (required)
        Enumeration of binary comparison operator applied between static value and incoming stream.
        Enum values are mapped to operators like '==', '<', and '<=', etc.
    value :  (required)
        Static value to compare against incoming stream using operator specified in `op`.
    name : str
        Name of the constraint used for reporting
    verbose : bool
        If true, log every application of this constraint that fails.
        Useful to identify specific streaming values that fail the constraint.

    """

    def __init__(self, op: Op, value=None, regex_pattern: str = None, name: str = None, verbose=False):
        self._name = name
        self._verbose = verbose
        self.op = op
        self.total = 0
        self.failures = 0
        if value is not None and regex_pattern is None:
            # numeric value
            self.value = value
            self.func = _value_funcs[op](value)

        elif regex_pattern is not None and value is None:
            # Regex pattern
            self.regex_pattern = regex_pattern
            self.func = _value_funcs[op](re.compile(self.regex_pattern))

        else:
            raise ValueError("Value constraint must specify a numeric value or regex pattern, but not both")

    @property
    def name(self):
        if getattr(self, "value", None):
            return self._name if self._name is not None else f"value {Op.Name(self.op)} {self.value}"
        else:
            return self._name if self._name is not None else f"value {Op.Name(self.op)} {self.regex_pattern}"

    def update(self, v) -> bool:
        self.total += 1
        if self.op in [Op.MATCH, Op.NOMATCH] and not isinstance(v, str):
            self.failures += 1
            if self._verbose:
                logger.info(f"value constraint {self.name} failed: value {v} not a string")
        elif not self.func(v):
            self.failures += 1
            if self._verbose:
                logger.info(f"value constraint {self.name} failed on value {v}")

    def merge(self, other) -> "ValueConstraint":
        if not other:
            return self
        assert self.name == other.name, f"Cannot merge constraints with different names: ({self.name}) and ({other.name})"
        assert self.op == other.op, f"Cannot merge constraints with different ops: {self.op} and {other.op}"
        assert self.value == other.value, f"Cannot merge value constraints with different values: {self.value} and {other.value}"
        merged_value_constraint = ValueConstraint(op=self.op, value=self.value, name=self.name, verbose=self._verbose)
        merged_value_constraint.total = self.total + other.total
        merged_value_constraint.failures = self.failures + other.failures
        return merged_value_constraint

    @staticmethod
    def from_protobuf(msg: ValueConstraintMsg) -> "ValueConstraint":
        return ValueConstraint(msg.op, msg.value, name=msg.name, verbose=msg.verbose)

    def to_protobuf(self) -> ValueConstraintMsg:
        if hasattr(self, "value"):
            return ValueConstraintMsg(
                name=self.name,
                op=self.op,
                value=self.value,
                verbose=self._verbose,
            )
        else:
            return ValueConstraintMsg(
                name=self.name,
                op=self.op,
                regex_pattern=self.regex_pattern,
                verbose=self._verbose,
            )

    def report(self):
        return (self.name, self.total, self.failures)


class SummaryConstraint:
    """
    Summary constraints specify a relationship between a summary field and a static value,
    or between two summary fields.
    e.g.     'min' < 6
             'std_dev' < 2.17
             'min' > 'avg'

    Parameters
    ----------
    first_field : str
        Name of field in NumberSummary that will be compared against either a second field or a static value.
    op : whylogs.proto.Op (required)
        Enumeration of binary comparison operator applied between summary values.
        Enum values are mapped to operators like '==', '<', and '<=', etc.
    value :  (one-of)
        Static value to be compared against summary field specified in `first_field`.
        Only one of `value` or `second_field` should be supplied.
    upper_value :  (one-of)
        Only to be supplied when using Op.BTWN.
        Static upper boundary value to be compared against summary field specified in `first_field`.
        Only one of `upper_value` or `third_field` should be supplied.
    second_field :  (one-of)
        Name of second field in NumberSummary to be compared against summary field specified in `first_field`.
        Only one of `value` or `second_field` should be supplied.
    third_field :  (one-of)
        Only to be supplied when op == Op.BTWN. Name of third field in NumberSummary, used as an upper boundary,
         to be compared against summary field specified in `first_field`.
        Only one of `upper_value` or `third_field` should be supplied.
    name : str
        Name of the constraint used for reporting
    verbose : bool
        If true, log every application of this constraint that fails.
        Useful to identify specific streaming values that fail the constraint.


    """

    def __init__(
        self,
        first_field: str,
        op: Op,
        value=None,
        upper_value=None,
        second_field: str = None,
        third_field: str = None,
        name: str = None,
        verbose=False,
    ):
        self._verbose = verbose
        self._name = name
        self.op = op
        self.first_field = first_field
        self.second_field = second_field
        self.third_field = third_field
        self.total = 0
        self.failures = 0

        self.value = value
        self.upper_value = upper_value

        if self.op == Op.BTWN:
            if value is not None and upper_value is not None and (second_field, third_field) == (None, None):
                # field-value summary comparison
                if not isinstance(value, (int, float)) or not isinstance(upper_value, (int, float)):
                    raise TypeError("When creating Summary constraint with BETWEEN operation, upper and lower value must be of type (int, float)")
                if value >= upper_value:
                    raise ValueError("Summary constraint with BETWEEN operation must specify lower value to be less than upper value")

                self.func = _summary_funcs1[self.op](first_field, value, upper_value)

            elif second_field is not None and third_field is not None and (value, upper_value) == (None, None):
                # field-field summary comparison
                if not isinstance(second_field, str) or not isinstance(third_field, str):
                    raise TypeError("When creating Summary constraint with BETWEEN operation, upper and lower field must be of type string")

                self.func = _summary_funcs2[self.op](first_field, second_field, third_field)
            else:
                raise ValueError("Summary constraint with BETWEEN operation must specify lower and upper value OR lower and third field name, but not both")
        else:
            if upper_value is not None or third_field is not None:
                raise ValueError("Summary constraint with other than BETWEEN operation must NOT specify upper value NOR third field name")

            if value is not None and second_field is None:
                # field-value summary comparison

                self.func = _summary_funcs1[op](first_field, value)
            elif second_field is not None and value is None:
                # field-field summary comparison

                self.func = _summary_funcs2[op](first_field, second_field)
            else:
                raise ValueError("Summary constraint must specify a second value or field name, but not both")

    @property
    def name(self):
        if self.op == Op.BTWN:
            lower_target = self.value if self.value is not None else self.second_field
            upper_target = self.upper_value if self.upper_value is not None else self.third_field
            return self._name if self._name is not None else f"summary {self.first_field} {Op.Name(self.op)} {lower_target} and {upper_target}"

        return self._name if self._name is not None else f"summary {self.first_field} {Op.Name(self.op)} {self.value}/{self.second_field}"

    def update(self, summ: NumberSummary) -> bool:
        self.total += 1
        if not self.func(summ):
            self.failures += 1
            if self._verbose:
                logger.info(f"summary constraint {self.name} failed")

    def merge(self, other) -> "SummaryConstraint":
        if not other:
            return self

        assert self.name == other.name, f"Cannot merge constraints with different names: ({self.name}) and ({other.name})"
        assert self.op == other.op, f"Cannot merge constraints with different ops: {self.op} and {other.op}"
        assert self.value == other.value, f"Cannot merge constraints with different values: {self.value} and {other.value}"
        assert self.first_field == other.first_field, f"Cannot merge constraints with different first_field: {self.first_field} and {other.first_field}"
        assert self.second_field == other.second_field, f"Cannot merge constraints with different second_field: {self.second_field} and {other.second_field}"

        if self.op == Op.BTWN:
            assert self.upper_value == other.upper_value, f"Cannot merge constraints with different upper values: {self.upper_value} and {other.upper_value}"
            assert self.third_field == other.third_field, f"Cannot merge constraints with different third_field: {self.third_field} and {other.third_field}"
            merged_constraint = SummaryConstraint(
                first_field=self.first_field,
                op=self.op,
                value=self.value,
                upper_value=self.upper_value,
                second_field=self.second_field,
                third_field=self.third_field,
                name=self.name,
                verbose=self._verbose,
            )
        else:
            merged_constraint = SummaryConstraint(
                first_field=self.first_field, op=self.op, value=self.value, second_field=self.second_field, name=self.name, verbose=self._verbose
            )

        merged_constraint.total = self.total + other.total
        merged_constraint.failures = self.failures + other.failures
        return merged_constraint

    @staticmethod
    def from_protobuf(msg: SummaryConstraintMsg) -> "SummaryConstraint":

        if msg.HasField("value") and not msg.HasField("second_field") and not msg.HasField("between"):
            return SummaryConstraint(
                msg.first_field,
                msg.op,
                value=msg.value,
                name=msg.name,
                verbose=msg.verbose,
            )
        elif msg.HasField("second_field") and not msg.HasField("value") and not msg.HasField("between"):
            return SummaryConstraint(
                msg.first_field,
                msg.op,
                second_field=msg.second_field,
                name=msg.name,
                verbose=msg.verbose,
            )
        elif msg.HasField("between") and not msg.HasField("value") and not msg.HasField("second_field"):
            if (
                msg.between.HasField("lower_value")
                and msg.between.HasField("upper_value")
                and not msg.between.HasField("second_field")
                and not msg.between.HasField("third_field")
            ):
                return SummaryConstraint(
                    msg.first_field,
                    msg.op,
                    value=msg.between.lower_value,
                    upper_value=msg.between.upper_value,
                    name=msg.name,
                    verbose=msg.verbose,
                )
            elif (
                msg.between.HasField("second_field")
                and msg.between.HasField("third_field")
                and not msg.between.HasField("lower_value")
                and not msg.between.HasField("upper_value")
            ):
                return SummaryConstraint(
                    msg.first_field,
                    msg.op,
                    second_field=msg.between.second_field,
                    third_field=msg.between.third_field,
                    name=msg.name,
                    verbose=msg.verbose,
                )
        else:
            raise ValueError("SummaryConstraintMsg must specify a value OR second field name OR SummaryBetweenConstraintMsg, but only one of them")

    def to_protobuf(self) -> SummaryConstraintMsg:
        if self.op == Op.BTWN:

            summary_between_constraint_msg = None
            if self.second_field is None and self.third_field is None:
                summary_between_constraint_msg = SummaryBetweenConstraintMsg(lower_value=self.value, upper_value=self.upper_value)
            else:
                summary_between_constraint_msg = SummaryBetweenConstraintMsg(second_field=self.second_field, third_field=self.third_field)

            msg = SummaryConstraintMsg(
                name=self.name,
                first_field=self.first_field,
                op=self.op,
                between=summary_between_constraint_msg,
                verbose=self._verbose,
            )

        elif self.second_field is None:
            msg = SummaryConstraintMsg(
                name=self.name,
                first_field=self.first_field,
                op=self.op,
                value=self.value,
                verbose=self._verbose,
            )
        else:
            msg = SummaryConstraintMsg(
                name=self.name,
                first_field=self.first_field,
                op=self.op,
                second_field=self.second_field,
                verbose=self._verbose,
            )
        return msg

    def report(self):
        return (self.name, self.total, self.failures)


class ValueConstraints:
    def __init__(self, constraints: Mapping[str, ValueConstraint] = None):
        if constraints is None:
            constraints = dict()

        if isinstance(constraints, list):
            self.constraints = {constraint.name: constraint for constraint in constraints}
        else:
            self.constraints = constraints

    @staticmethod
    def from_protobuf(msg: ValueConstraintMsgs) -> "ValueConstraints":
        value_constraints = [ValueConstraint.from_protobuf(c) for c in msg.constraints]
        if len(value_constraints) > 0:
            return ValueConstraints({v.name: v for v in value_constraints})
        return None

    def __getitem__(self, name: str) -> Optional[ValueConstraint]:
        if self.contraints:
            return self.constraints.get(name)
        return None

    def to_protobuf(self) -> ValueConstraintMsgs:
        v = [c.to_protobuf() for c in self.constraints.values()]
        if len(v) > 0:
            vcmsg = ValueConstraintMsgs()
            vcmsg.constraints.extend(v)
            return vcmsg
        return None

    def update(self, v):
        for c in self.constraints.values():
            c.update(v)

    def merge(self, other) -> "ValueConstraints":
        if not other or not other.constraints:
            return self

        merged_constraints = other.constraints.copy()
        for name, constraint in self.constraints:
            merged_constraints[name] = constraint.merge(other.constraints.get(name))

        return ValueConstraints(merged_constraints)

    def report(self) -> List[tuple]:
        v = [c.report() for c in self.constraints.values()]
        if len(v) > 0:
            return v
        return None


class SummaryConstraints:
    def __init__(self, constraints: Mapping[str, SummaryConstraint] = None):
        if constraints is None:
            constraints = dict()

        # Support list of constraints for back compat with previous version.
        if isinstance(constraints, list):
            self.constraints = {constraint.name: constraint for constraint in constraints}
        else:
            self.constraints = constraints

    @staticmethod
    def from_protobuf(msg: SummaryConstraintMsgs) -> "SummaryConstraints":
        constraints = [SummaryConstraint.from_protobuf(c) for c in msg.constraints]
        if len(constraints) > 0:
            return SummaryConstraints({v.name: v for v in constraints})
        return None

    def __getitem__(self, name: str) -> Optional[SummaryConstraint]:
        if self.contraints:
            return self.constraints.get(name)
        return None

    def to_protobuf(self) -> SummaryConstraintMsgs:
        v = [c.to_protobuf() for c in self.constraints.values()]
        if len(v) > 0:
            scmsg = SummaryConstraintMsgs()
            scmsg.constraints.extend(v)
            return scmsg
        return None

    def update(self, v):
        for c in self.constraints.values():
            c.update(v)

    def merge(self, other) -> "SummaryConstraints":

        if not other or not other.constraints:
            return self

        merged_constraints = other.constraints.copy()
        for name, constraint in self.constraints:
            merged_constraints[name] = constraint.merge(other.constraints.get(name))

        return SummaryConstraints(merged_constraints)

    def report(self) -> List[tuple]:
        v = [c.report() for c in self.constraints.values()]
        if len(v) > 0:
            return v
        return None


class DatasetConstraints:
    def __init__(
        self,
        props: DatasetProperties,
        value_constraints: Optional[ValueConstraints] = None,
        summary_constraints: Optional[SummaryConstraints] = None,
    ):
        self.dataset_properties = props
        # repackage lists of constraints if necessary
        if value_constraints is None:
            value_constraints = dict()
        for k, v in value_constraints.items():
            if isinstance(v, list):
                value_constraints[k] = ValueConstraints(v)
        self.value_constraint_map = value_constraints
        if summary_constraints is None:
            summary_constraints = dict()
        for k, v in summary_constraints.items():
            if isinstance(v, list):
                summary_constraints[k] = SummaryConstraints(v)
        self.summary_constraint_map = summary_constraints

    def __getitem__(self, key):
        if key in self.value_constraint_map:
            return self.value_constraint_map[key]
        return None

    @staticmethod
    def from_protobuf(msg: DatasetConstraintMsg) -> "DatasetConstraints":
        vm = dict([(k, ValueConstraints.from_protobuf(v)) for k, v in msg.value_constraints.items()])
        sm = dict([(k, SummaryConstraints.from_protobuf(v)) for k, v in msg.summary_constraints.items()])
        return DatasetConstraints(msg.properties, vm, sm)

    @staticmethod
    def from_json(data: str) -> "DatasetConstraints":
        msg = Parse(data, DatasetConstraintMsg())
        return DatasetConstraints.from_protobuf(msg)

    def to_protobuf(self) -> DatasetConstraintMsg:
        # construct tuple for each column, (name, [constraints,...])
        # turn that into a map indexed by column name
        vm = dict([(k, v.to_protobuf()) for k, v in self.value_constraint_map.items()])
        sm = dict([(k, s.to_protobuf()) for k, s in self.summary_constraint_map.items()])
        return DatasetConstraintMsg(
            properties=self.dataset_properties,
            value_constraints=vm,
            summary_constraints=sm,
        )

    def to_json(self) -> str:
        return message_to_json(self.to_protobuf())

    def report(self):
        l1 = [(k, v.report()) for k, v in self.value_constraint_map.items()]
        l2 = [(k, s.report()) for k, s in self.summary_constraint_map.items()]
        return l1 + l2


def stddevBetweenConstraint(lower_value=None, upper_value=None, lower_field=None, upper_field=None, verbose=False):
    return SummaryConstraint("stddev", Op.BTWN, value=lower_value, upper_value=upper_value, second_field=lower_field, third_field=upper_field, verbose=verbose)


def meanBetweenConstraint(lower_value=None, upper_value=None, lower_field=None, upper_field=None, verbose=False):
    return SummaryConstraint("mean", Op.BTWN, value=lower_value, upper_value=upper_value, second_field=lower_field, third_field=upper_field, verbose=verbose)


def minBetweenConstraint(lower_value=None, upper_value=None, lower_field=None, upper_field=None, verbose=False):
    return SummaryConstraint("min", Op.BTWN, value=lower_value, upper_value=upper_value, second_field=lower_field, third_field=upper_field, verbose=verbose)


def minGreaterThanEqualConstraint(value=None, field=None, verbose=False):
    return SummaryConstraint("min", Op.GE, value=value, second_field=field, verbose=verbose)


def maxBetweenConstraint(lower_value=None, upper_value=None, lower_field=None, upper_field=None, verbose=False):
    return SummaryConstraint("max", Op.BTWN, value=lower_value, upper_value=upper_value, second_field=lower_field, third_field=upper_field, verbose=verbose)


def maxLessThanEqualConstraint(value=None, field=None, verbose=False):
    return SummaryConstraint("max", Op.LE, value=value, second_field=field, verbose=verbose)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0emessages.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\x7f\n\x08\x43ounters\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12/\n\ntrue_count\x18\x02 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\x12\x33\n\nnull_count\x18\x03 \x01(\x0b\x32\x1b.google.protobuf.Int64ValueB\x02\x18\x01\"\x95\x01\n\x0cInferredType\x12 \n\x04type\x18\x01 \x01(\x0e\x32\x12.InferredType.Type\x12\r\n\x05ratio\x18\x02 \x01(\x01\"T\n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04NULL\x10\x01\x12\x0e\n\nFRACTIONAL\x10\x02\x12\x0c\n\x08INTEGRAL\x10\x03\x12\x0b\n\x07\x42OOLEAN\x10\x04\x12\n\n\x06STRING\x10\x05\"F\n\x0e\x44oublesMessage\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12\x0b\n\x03min\x18\x02 \x01(\x01\x12\x0b\n\x03max\x18\x03 \x01(\x01\x12\x0b\n\x03sum\x18\x04 \x01(\x01\"D\n\x0cLongsMessage\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12\x0b\n\x03min\x18\x02 \x01(\x03\x12\x0b\n\x03max\x18\x03 \x01(\x03\x12\x0b\n\x03sum\x18\x04 \x01(\x03\";\n\x0fVarianceMessage\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12\x0b\n\x03sum\x18\x02 \x01(\x01\x12\x0c\n\x04mean\x18\x03 \x01(\x01\"@\n\x1c\x46requentNumbersSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\x12\x10\n\x08lg_max_k\x18\x02 \x01(\x05\">\n\x1a\x46requentItemsSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\x12\x10\n\x08lg_max_k\x18\x02 \x01(\x05\"\xf5\x01\n\x0eNumbersMessage\x12\"\n\x08variance\x18\x01 \x01(\x0b\x32\x10.VarianceMessage\x12\"\n\x07\x64oubles\x18\x02 \x01(\x0b\x32\x0f.DoublesMessageH\x00\x12\x1e\n\x05longs\x18\x03 \x01(\x0b\x32\r.LongsMessageH\x00\x12\x11\n\thistogram\x18\x04 \x01(\x0c\x12\r\n\x05theta\x18\x05 \x01(\x0c\x12\x15\n\rcompact_theta\x18\x06 \x01(\x0c\x12\x37\n\x10\x66requent_numbers\x18\x07 \x01(\x0b\x32\x1d.FrequentNumbersSketchMessageB\t\n\x07numbers\"\x9e\x01\n\x0e\x43harPosMessage\x12\x11\n\tchar_list\x18\x01 \x01(\t\x12\x35\n\x0c\x63har_pos_map\x18\x02 \x03(\x0b\x32\x1f.CharPosMessage.CharPosMapEntry\x1a\x42\n\x0f\x43harPosMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.NumbersMessage:\x02\x38\x01\"\xc7\x01\n\x0eStringsMessage\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12\r\n\x05theta\x18\x02 \x01(\x0c\x12\r\n\x05items\x18\x03 \x01(\x0c\x12\x15\n\rcompact_theta\x18\x04 \x01(\x0c\x12\x1f\n\x06length\x18\x05 \x01(\x0b\x32\x0f.NumbersMessage\x12%\n\x0ctoken_length\x18\x06 \x01(\x0b\x32\x0f.NumbersMessage\x12)\n\x10\x63har_pos_tracker\x18\x07 \x01(\x0b\x32\x0f.CharPosMessage\"\x9c\x01\n\rSchemaMessage\x12\x32\n\ntypeCounts\x18\x01 \x03(\x0b\x32\x1e.SchemaMessage.TypeCountsEntry\x12$\n\rinferred_type\x18\x02 \x01(\x0b\x32\r.InferredType\x1a\x31\n\x0fTypeCountsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"\xa9\x02\n\rColumnMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1b\n\x08\x63ounters\x18\x02 \x01(\x0b\x32\t.Counters\x12\x1e\n\x06schema\x18\x03 \x01(\x0b\x32\x0e.SchemaMessage\x12 \n\x07numbers\x18\x04 \x01(\x0b\x32\x0f.NumbersMessage\x12 \n\x07strings\x18\x05 \x01(\x0b\x32\x0f.StringsMessage\x12$\n\rinferred_type\x18\x06 \x01(\x0b\x32\r.InferredType\x12\x33\n\x0e\x66requent_items\x18\x07 \x01(\x0b\x32\x1b.FrequentItemsSketchMessage\x12.\n\x13\x63\x61rdinality_tracker\x18\x08 \x01(\x0b\x32\x11.HllSketchMessage\"\xd4\x02\n\x11\x44\x61tasetProperties\x12\x1c\n\x14schema_major_version\x18\x01 \x01(\r\x12\x1c\n\x14schema_minor_version\x18\x02 \x01(\r\x12\x12\n\nsession_id\x18\x03 \x01(\t\x12\x19\n\x11session_timestamp\x18\x04 \x01(\x03\x12\x16\n\x0e\x64\x61ta_timestamp\x18\x05 \x01(\x03\x12*\n\x04tags\x18\x06 \x03(\x0b\x32\x1c.DatasetProperties.TagsEntry\x12\x32\n\x08metadata\x18\x07 \x03(\x0b\x32 .DatasetProperties.MetadataEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8a\x01\n\x12ScoreMatrixMessage\x12\x0e\n\x06labels\x18\x01 \x03(\t\x12\x18\n\x10prediction_field\x18\x02 \x01(\t\x12\x14\n\x0ctarget_field\x18\x03 \x01(\t\x12\x13\n\x0bscore_field\x18\x04 \x01(\t\x12\x1f\n\x06scores\x18\n \x03(\x0b\x32\x0f.NumbersMessage\"\x94\x01\n\x18RegressionMetricsMessage\x12\x18\n\x10prediction_field\x18\x01 \x01(\t\x12\x14\n\x0ctarget_field\x18\x02 \x01(\t\x12\r\n\x05\x63ount\x18\x03 \x01(\x04\x12\x14\n\x0csum_abs_diff\x18\x04 \x01(\x01\x12\x10\n\x08sum_diff\x18\x05 \x01(\x01\x12\x11\n\tsum2_diff\x18\x06 \x01(\x01\"m\n\x11NLPMetricsMessage\x12\x1c\n\x03mer\x18\x01 \x01(\x0b\x32\x0f.NumbersMessage\x12\x1c\n\x03wer\x18\x02 \x01(\x0b\x32\x0f.NumbersMessage\x12\x1c\n\x03wil\x18\x03 \x01(\x0b\x32\x0f.NumbersMessage\"\x94\x01\n\x13ModelMetricsMessage\x12(\n\x0bscoreMatrix\x18\x01 \x01(\x0b\x32\x13.ScoreMatrixMessage\x12\x1d\n\tmodelType\x18\x02 \x01(\x0e\x32\n.ModelType\x12\x34\n\x11regressionMetrics\x18\x03 \x01(\x0b\x32\x19.RegressionMetricsMessage\"S\n\x13ModelProfileMessage\x12\x15\n\routput_fields\x18\x01 \x03(\t\x12%\n\x07metrics\x18\n \x01(\x0b\x32\x14.ModelMetricsMessage\"\xe0\x01\n\x15\x44\x61tasetProfileMessage\x12&\n\nproperties\x18\x01 \x01(\x0b\x32\x12.DatasetProperties\x12\x34\n\x07\x63olumns\x18\x02 \x03(\x0b\x32#.DatasetProfileMessage.ColumnsEntry\x12)\n\x0bmodeProfile\x18\n \x01(\x0b\x32\x14.ModelProfileMessage\x1a>\n\x0c\x43olumnsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1d\n\x05value\x18\x02 \x01(\x0b\x32\x0e.ColumnMessage:\x02\x38\x01\"F\n\x13\x43olumnsChunkSegment\x12\x0e\n\x06marker\x18\x01 \x01(\t\x12\x1f\n\x07\x63olumns\x18\x02 \x03(\x0b\x32\x0e.ColumnMessage\"P\n\x16\x44\x61tasetMetadataSegment\x12\x0e\n\x06marker\x18\x01 \x01(\t\x12&\n\nproperties\x18\x02 \x01(\x0b\x32\x12.DatasetProperties\"~\n\x0eMessageSegment\x12\x0e\n\x06marker\x18\x01 \x01(\t\x12+\n\x08metadata\x18\x02 \x01(\x0b\x32\x17.DatasetMetadataSegmentH\x00\x12\'\n\x07\x63olumns\x18\x03 \x01(\x0b\x32\x14.ColumnsChunkSegmentH\x00\x42\x06\n\x04item\"0\n\x10HllSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\x12\x0c\n\x04lg_k\x18\x02 \x01(\x05*U\n\tModelType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x12\n\x0e\x43LASSIFICATION\x10\x01\x12\x0e\n\nREGRESSION\x10\x02\x12\x0e\n\nEMBEDDINGS\x10\x03\x12\x07\n\x03NLP\x10\x04\x42&\n\x18\x63om.whylogs.core.messageB\x08MessagesP\x01\x62\x06proto3')

_MODELTYPE = DESCRIPTOR.enum_types_by_name['ModelType']
ModelType = enum_type_wrapper.EnumTypeWrapper(_MODELTYPE)
UNKNOWN = 0
CLASSIFICATION = 1
REGRESSION = 2
EMBEDDINGS = 3
NLP = 4


_COUNTERS = DESCRIPTOR.message_types_by_name['Counters']
_INFERREDTYPE = DESCRIPTOR.message_types_by_name['InferredType']
_DOUBLESMESSAGE = DESCRIPTOR.message_types_by_name['DoublesMessage']
_LONGSMESSAGE = DESCRIPTOR.message_types_by_name['LongsMessage']
_VARIANCEMESSAGE = DESCRIPTOR.message_types_by_name['VarianceMessage']
_FREQUENTNUMBERSSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['FrequentNumbersSketchMessage']
_FREQUENTITEMSSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['FrequentItemsSketchMessage']
_NUMBERSMESSAGE = DESCRIPTOR.message_types_by_name['NumbersMessage']
_CHARPOSMESSAGE = DESCRIPTOR.message_types_by_name['CharPosMessage']
_CHARPOSMESSAGE_CHARPOSMAPENTRY = _CHARPOSMESSAGE.nested_types_by_name['CharPosMapEntry']
_STRINGSMESSAGE = DESCRIPTOR.message_types_by_name['StringsMessage']
_SCHEMAMESSAGE = DESCRIPTOR.message_types_by_name['SchemaMessage']
_SCHEMAMESSAGE_TYPECOUNTSENTRY = _SCHEMAMESSAGE.nested_types_by_name['TypeCountsEntry']
_COLUMNMESSAGE = DESCRIPTOR.message_types_by_name['ColumnMessage']
_DATASETPROPERTIES = DESCRIPTOR.message_types_by_name['DatasetProperties']
_DATASETPROPERTIES_TAGSENTRY = _DATASETPROPERTIES.nested_types_by_name['TagsEntry']
_DATASETPROPERTIES_METADATAENTRY = _DATASETPROPERTIES.nested_types_by_name['MetadataEntry']
_SCOREMATRIXMESSAGE = DESCRIPTOR.message_types_by_name['ScoreMatrixMessage']
_REGRESSIONMETRICSMESSAGE = DESCRIPTOR.message_types_by_name['RegressionMetricsMessage']
_NLPMETRICSMESSAGE = DESCRIPTOR.message_types_by_name['NLPMetricsMessage']
_MODELMETRICSMESSAGE = DESCRIPTOR.message_types_by_name['ModelMetricsMessage']
_MODELPROFILEMESSAGE = DESCRIPTOR.message_types_by_name['ModelProfileMessage']
_DATASETPROFILEMESSAGE = DESCRIPTOR.message_types_by_name['DatasetProfileMessage']
_DATASETPROFILEMESSAGE_COLUMNSENTRY = _DATASETPROFILEMESSAGE.nested_types_by_name['ColumnsEntry']
_COLUMNSCHUNKSEGMENT = DESCRIPTOR.message_types_by_name['ColumnsChunkSegment']
_DATASETMETADATASEGMENT = DESCRIPTOR.message_types_by_name['DatasetMetadataSegment']
_MESSAGESEGMENT = DESCRIPTOR.message_types_by_name['MessageSegment']
_HLLSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['HllSketchMessage']
_INFERREDTYPE_TYPE = _INFERREDTYPE.enum_types_by_name['Type']
Counters = _reflection.GeneratedProtocolMessageType('Counters', (_message.Message,), {
  'DESCRIPTOR' : _COUNTERS,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:Counters)
  })
_sym_db.RegisterMessage(Counters)

InferredType = _reflection.GeneratedProtocolMessageType('InferredType', (_message.Message,), {
  'DESCRIPTOR' : _INFERREDTYPE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:InferredType)
  })
_sym_db.RegisterMessage(InferredType)

DoublesMessage = _reflection.GeneratedProtocolMessageType('DoublesMessage', (_message.Message,), {
  'DESCRIPTOR' : _DOUBLESMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:DoublesMessage)
  })
_sym_db.RegisterMessage(DoublesMessage)

LongsMessage = _reflection.GeneratedProtocolMessageType('LongsMessage', (_message.Message,), {
  'DESCRIPTOR' : _LONGSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:LongsMessage)
  })
_sym_db.RegisterMessage(LongsMessage)

VarianceMessage = _reflection.GeneratedProtocolMessageType('VarianceMessage', (_message.Message,), {
  'DESCRIPTOR' : _VARIANCEMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:VarianceMessage)
  })
_sym_db.RegisterMessage(VarianceMessage)

FrequentNumbersSketchMessage = _reflection.GeneratedProtocolMessageType('FrequentNumbersSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _FREQUENTNUMBERSSKETCHMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:FrequentNumbersSketchMessage)
  })
_sym_db.RegisterMessage(FrequentNumbersSketchMessage)

FrequentItemsSketchMessage = _reflection.GeneratedProtocolMessageType('FrequentItemsSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _FREQUENTITEMSSKETCHMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:FrequentItemsSketchMessage)
  })
_sym_db.RegisterMessage(FrequentItemsSketchMessage)

NumbersMessage = _reflection.GeneratedProtocolMessageType('NumbersMessage', (_message.Message,), {
  'DESCRIPTOR' : _NUMBERSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:NumbersMessage)
  })
_sym_db.RegisterMessage(NumbersMessage)

CharPosMessage = _reflection.GeneratedProtocolMessageType('CharPosMessage', (_message.Message,), {

  'CharPosMapEntry' : _reflection.GeneratedProtocolMessageType('CharPosMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _CHARPOSMESSAGE_CHARPOSMAPENTRY,
    '__module__' : 'messages_pb2'
    # @@protoc_insertion_point(class_scope:CharPosMessage.CharPosMapEntry)
    })
  ,
  'DESCRIPTOR' : _CHARPOSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:CharPosMessage)
  })
_sym_db.RegisterMessage(CharPosMessage)
_sym_db.RegisterMessage(CharPosMessage.CharPosMapEntry)

StringsMessage = _reflection.GeneratedProtocolMessageType('StringsMessage', (_message.Message,), {
  'DESCRIPTOR' : _STRINGSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:StringsMessage)
  })
_sym_db.RegisterMessage(StringsMessage)

SchemaMessage = _reflection.GeneratedProtocolMessageType('SchemaMessage', (_message.Message,), {

  'TypeCountsEntry' : _reflection.GeneratedProtocolMessageType('TypeCountsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SCHEMAMESSAGE_TYPECOUNTSENTRY,
    '__module__' : 'messages_pb2'
    # @@protoc_insertion_point(class_scope:SchemaMessage.TypeCountsEntry)
    })
  ,
  'DESCRIPTOR' : _SCHEMAMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:SchemaMessage)
  })
_sym_db.RegisterMessage(SchemaMessage)
_sym_db.RegisterMessage(SchemaMessage.TypeCountsEntry)

ColumnMessage = _reflection.GeneratedProtocolMessageType('ColumnMessage', (_message.Message,), {
  'DESCRIPTOR' : _COLUMNMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ColumnMessage)
  })
_sym_db.RegisterMessage(ColumnMessage)

DatasetProperties = _reflection.GeneratedProtocolMessageType('DatasetProperties', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROPERTIES_TAGSENTRY,
    '__module__' : 'messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProperties.TagsEntry)
    })
  ,

  'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROPERTIES_METADATAENTRY,
    '__module__' : 'messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProperties.MetadataEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETPROPERTIES,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:DatasetProperties)
  })
_sym_db.RegisterMessage(DatasetProperties)
_sym_db.RegisterMessage(DatasetProperties.TagsEntry)
_sym_db.RegisterMessage(DatasetProperties.MetadataEntry)

ScoreMatrixMessage = _reflection.GeneratedProtocolMessageType('ScoreMatrixMessage', (_message.Message,), {
  'DESCRIPTOR' : _SCOREMATRIXMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ScoreMatrixMessage)
  })
_sym_db.RegisterMessage(ScoreMatrixMessage)

RegressionMetricsMessage = _reflection.GeneratedProtocolMessageType('RegressionMetricsMessage', (_message.Message,), {
  'DESCRIPTOR' : _REGRESSIONMETRICSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:RegressionMetricsMessage)
  })
_sym_db.RegisterMessage(RegressionMetricsMessage)

NLPMetricsMessage = _reflection.GeneratedProtocolMessageType('NLPMetricsMessage', (_message.Message,), {
  'DESCRIPTOR' : _NLPMETRICSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:NLPMetricsMessage)
  })
_sym_db.RegisterMessage(NLPMetricsMessage)

ModelMetricsMessage = _reflection.GeneratedProtocolMessageType('ModelMetricsMessage', (_message.Message,), {
  'DESCRIPTOR' : _MODELMETRICSMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ModelMetricsMessage)
  })
_sym_db.RegisterMessage(ModelMetricsMessage)

ModelProfileMessage = _reflection.GeneratedProtocolMessageType('ModelProfileMessage', (_message.Message,), {
  'DESCRIPTOR' : _MODELPROFILEMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ModelProfileMessage)
  })
_sym_db.RegisterMessage(ModelProfileMessage)

DatasetProfileMessage = _reflection.GeneratedProtocolMessageType('DatasetProfileMessage', (_message.Message,), {

  'ColumnsEntry' : _reflection.GeneratedProtocolMessageType('ColumnsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROFILEMESSAGE_COLUMNSENTRY,
    '__module__' : 'messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProfileMessage.ColumnsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETPROFILEMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:DatasetProfileMessage)
  })
_sym_db.RegisterMessage(DatasetProfileMessage)
_sym_db.RegisterMessage(DatasetProfileMessage.ColumnsEntry)

ColumnsChunkSegment = _reflection.GeneratedProtocolMessageType('ColumnsChunkSegment', (_message.Message,), {
  'DESCRIPTOR' : _COLUMNSCHUNKSEGMENT,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ColumnsChunkSegment)
  })
_sym_db.RegisterMessage(ColumnsChunkSegment)

DatasetMetadataSegment = _reflection.GeneratedProtocolMessageType('DatasetMetadataSegment', (_message.Message,), {
  'DESCRIPTOR' : _DATASETMETADATASEGMENT,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:DatasetMetadataSegment)
  })
_sym_db.RegisterMessage(DatasetMetadataSegment)

MessageSegment = _reflection.GeneratedProtocolMessageType('MessageSegment', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGESEGMENT,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:MessageSegment)
  })
_sym_db.RegisterMessage(MessageSegment)

HllSketchMessage = _reflection.GeneratedProtocolMessageType('HllSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _HLLSKETCHMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:HllSketchMessage)
  })
_sym_db.RegisterMessage(HllSketchMessage)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.whylogs.core.messageB\010MessagesP\001'
  _COUNTERS.fields_by_name['null_count']._options = None
  _COUNTERS.fields_by_name['null_count']._serialized_options = b'\030\001'
  _CHARPOSMESSAGE_CHARPOSMAPENTRY._options = None
  _CHARPOSMESSAGE_CHARPOSMAPENTRY._serialized_options = b'8\001'
  _SCHEMAMESSAGE_TYPECOUNTSENTRY._options = None
  _SCHEMAMESSAGE_TYPECOUNTSENTRY._serialized_options = b'8\001'
  _DATASETPROPERTIES_TAGSENTRY._options = None
  _DATASETPROPERTIES_TAGSENTRY._serialized_options = b'8\001'
  _DATASETPROPERTIES_METADATAENTRY._options = None
  _DATASETPROPERTIES_METADATAENTRY._serialized_options = b'8\001'
  _DATASETPROFILEMESSAGE_COLUMNSENTRY._options = None
  _DATASETPROFILEMESSAGE_COLUMNSENTRY._serialized_options = b'8\001'
  _MODELTYPE._serialized_start=3275
  _MODELTYPE._serialized_end=3360
  _COUNTERS._serialized_start=50
  _COUNTERS._serialized_end=177
  _INFERREDTYPE._serialized_start=180
  _INFERREDTYPE._serialized_end=329
  _INFERREDTYPE_TYPE._serialized_start=245
  _INFERREDTYPE_TYPE._serialized_end=329
  _DOUBLESMESSAGE._serialized_start=331
  _DOUBLESMESSAGE._serialized_end=401
  _LONGSMESSAGE._serialized_start=403
  _LONGSMESSAGE._serialized_end=471
  _VARIANCEMESSAGE._serialized_start=473
  _VARIANCEMESSAGE._serialized_end=532
  _FREQUENTNUMBERSSKETCHMESSAGE._serialized_start=534
  _FREQUENTNUMBERSSKETCHMESSAGE._serialized_end=598
  _FREQUENTITEMSSKETCHMESSAGE._serialized_start=600
  _FREQUENTITEMSSKETCHMESSAGE._serialized_end=662
  _NUMBERSMESSAGE._serialized_start=665
  _NUMBERSMESSAGE._serialized_end=910
  _CHARPOSMESSAGE._serialized_start=913
  _CHARPOSMESSAGE._serialized_end=1071
  _CHARPOSMESSAGE_CHARPOSMAPENTRY._serialized_start=1005
  _CHARPOSMESSAGE_CHARPOSMAPENTRY._serialized_end=1071
  _STRINGSMESSAGE._serialized_start=1074
  _STRINGSMESSAGE._serialized_end=1273
  _SCHEMAMESSAGE._serialized_start=1276
  _SCHEMAMESSAGE._serialized_end=1432
  _SCHEMAMESSAGE_TYPECOUNTSENTRY._serialized_start=1383
  _SCHEMAMESSAGE_TYPECOUNTSENTRY._serialized_end=1432
  _COLUMNMESSAGE._serialized_start=1435
  _COLUMNMESSAGE._serialized_end=1732
  _DATASETPROPERTIES._serialized_start=1735
  _DATASETPROPERTIES._serialized_end=2075
  _DATASETPROPERTIES_TAGSENTRY._serialized_start=1983
  _DATASETPROPERTIES_TAGSENTRY._serialized_end=2026
  _DATASETPROPERTIES_METADATAENTRY._serialized_start=2028
  _DATASETPROPERTIES_METADATAENTRY._serialized_end=2075
  _SCOREMATRIXMESSAGE._serialized_start=2078
  _SCOREMATRIXMESSAGE._serialized_end=2216
  _REGRESSIONMETRICSMESSAGE._serialized_start=2219
  _REGRESSIONMETRICSMESSAGE._serialized_end=2367
  _NLPMETRICSMESSAGE._serialized_start=2369
  _NLPMETRICSMESSAGE._serialized_end=2478
  _MODELMETRICSMESSAGE._serialized_start=2481
  _MODELMETRICSMESSAGE._serialized_end=2629
  _MODELPROFILEMESSAGE._serialized_start=2631
  _MODELPROFILEMESSAGE._serialized_end=2714
  _DATASETPROFILEMESSAGE._serialized_start=2717
  _DATASETPROFILEMESSAGE._serialized_end=2941
  _DATASETPROFILEMESSAGE_COLUMNSENTRY._serialized_start=2879
  _DATASETPROFILEMESSAGE_COLUMNSENTRY._serialized_end=2941
  _COLUMNSCHUNKSEGMENT._serialized_start=2943
  _COLUMNSCHUNKSEGMENT._serialized_end=3013
  _DATASETMETADATASEGMENT._serialized_start=3015
  _DATASETMETADATASEGMENT._serialized_end=3095
  _MESSAGESEGMENT._serialized_start=3097
  _MESSAGESEGMENT._serialized_end=3223
  _HLLSKETCHMESSAGE._serialized_start=3225
  _HLLSKETCHMESSAGE._serialized_end=3273
# @@protoc_insertion_point(module_scope)
